<?php
namespace Quivers\Integration\Controller\Index;
use Stripe\Stripe;
use Quivers\Integration\LogTracking;

class Index extends \Magento\Framework\App\Action\Action
{
    protected $_pageFactory;

    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\View\Result\PageFactory $pageFactory)
    {
        $this->_pageFactory = $pageFactory;
        $this->_checkoutSession = $checkoutSession;
        return parent::__construct($context);
        
    }
    
    const path = 'ecommerce';
    public function execute()
    {   
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $post = $this->getRequest()->getParam('input_val');
        $urlInterface = $objectManager->get('Magento\Framework\UrlInterface');
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $baseUrl=$storeManager->getStore()->getBaseUrl();
        if($baseUrl.self::path==$urlInterface->getCurrentUrl()){
            return  $this->_pageFactory->create();
        }
        $client_secret=$this->createPaymentIntent();
        $response = $this->resultFactory
            ->create(\Magento\Framework\Controller\ResultFactory::TYPE_JSON)
            ->setData([
                'status'  => "success",
                'client_secret' => isset($client_secret)?$client_secret:''
            ]);
        return $response;
    }

    public function createPaymentIntent()
    {
        try
        {   
            $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
            $log = new LogTracking($scopeConfig);
            $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
            $secret_key = $scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
            Stripe::setApiKey($secret_key);
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            #get stripe customer id
            $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
            $checkout_session=$objectManager->get('\Magento\Checkout\Model\Session');
            $customer=$customer_session->getCustomer();
            $customer_id=$customer->getQuiversStripeCustomerId();
            #get total anount 
            $grandTotal=$customer_session->getTotalAmount();
            $lastorder = $this->_checkoutSession->getLastRealOrder();
            $orderId = $lastorder->getEntityId();
            if(!$grandTotal){
                $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
                $cart = $objectManager->get('\Magento\Checkout\Model\Cart');
                $newGrandTotal = $cart->getQuote()->getGrandTotal();
                $totals = $cart->getQuote()->getTotals();
                $tax = $totals['tax']->getValue();
                $tax = 0.0;
                if(isset($totals['tax'])){
                    $tax = $totals['tax']->getValue();
                }
                $ExtraTax = $tax/2;
                $newGrandTotal = $newGrandTotal-$ExtraTax;
                
                if($newGrandTotal){
                    $grandTotal = $newGrandTotal;
                }
            }
            #get currency code
            $cart = $objectManager->get('\Magento\Checkout\Model\Cart');
            $quote = $checkout_session->getQuote();
            $currency= $cart->getQuote()->getQuoteCurrencyCode();
            #create charge amount
            $chargeAmount=$this->formatCurrencyAmount($grandTotal, $currency);
            #get  statement_descriptor_suffix
            $payment=$quote->getPayment();
            $statement_descriptor_suffix = $payment->getAdditionalInformation('payment_descriptor');
            if(strlen($statement_descriptor_suffix) > 10)
            {
                $statement_descriptor_suffix = substr($statement_descriptor_suffix, 0, 10);
            }
            #creating desciption
            $description=$quote->getBillingAddress()->getEmail();
            #creating payment intent payload
            $payload=array(
                    "amount" => $chargeAmount, 
                    "currency" => $currency,
                    "confirmation_method" => 'automatic',
                    "capture_method" => "manual",
                    "customer"=>isset($customer_id)?$customer_id:null,
                    "setup_future_usage"=>'off_session',
                    "description"=>$description,
                    "statement_descriptor_suffix"=>$statement_descriptor_suffix,

                );
            #get destination
            if($payment->getAdditionalInformation('destination') === true){
                $merchantId = $payment->getAdditionalInformation('merchantId');
                if($merchantId != '0'){
                    $transferData = array(
                        "destination" => $merchantId,
                        "amount" => $chargeAmount
                    );
                    $onBehalfOf = $merchantId;
                    $payload['transfer_data'] = $transferData;
                    $payload['on_behalf_of'] = $onBehalfOf;
                }
            }
            #creating payment intent
            $intent = \Stripe\PaymentIntent::create($payload);
            #set paymentintent id to session
            $customer_session->setPaymentIntentId($intent->id);
            return $intent->client_secret;
        }
        catch(\Exception $e)
        {
            $log->print_statement(json_encode($e));
               return  '';
        }
    }

    public function formatCurrencyAmount($amount, $currency)
    {

        // Stripe takes amounts as the smallest currency unit. ...
        if (empty($amount)) {
            return null;
        }
        $multiple = 100;
        if (in_array(
            $currency,
            array('BIF', 'DJF', 'JPY', 'KRW', 'PYG', 'VND', 'XAF', 'XPF', 'CLP', 'GNF', 'KMF', 'MGA', 'RWF', 'VUV', 'XOF'),
            true
        )) {
            $multiple = 1;
        }

        return sprintf('%01.0f', (float)$amount * $multiple);
    }
}
